.equ SENSOR_PORT = PORTC
.equ SENSOR_DDR  = DDRC
.equ SENSOR_PIN	= PINC
.equ PIN_DATA   = 2
.equ PIN_CLK    = 1

.equ Zahl 	= 4010 	//d1 fr Temperatur berechnen mit 10^3
.equ C1		= 2046	//Gerundet von 2.0468 auf 2.046*1000
.equ C21	= 36	//C2 aufgeteilt in vor dem Komma und nach dem Kommastellen 0.0367*10^3	 
.equ C22	= 7		//Nachkommastelle
.equ C23	= 10
.equ C31	= 1595	//C3 von 1.5955*10^-6 durch *10^3 auf 1.595 /1000 gerundet
.equ C32	= 1000
.equ C33	= 1000

.def	rhl=r8
.def	rhh=r9

.def	tl=r10
.def	th=r11

.def	rel=r12
.def	reh=r13

.def	trt=r14

.def	rhrt=r15

.ifndef XTAL
.equ XTAL = 4000000
.endif

  

;Kurze Pause        
shortbreak:                            ; 50us Pause
        ldi  temp1, ( XTAL * 50 / 3 ) / 1000000
shortbreak_:
        dec  temp1
        brne shortbreak_
        ret                         ; wieder zurck

dlow:	sbi SENSOR_DDR,PIN_DATA		;set data low
	rjmp rdelay
	
dhigh:	cbi SENSOR_DDR,PIN_DATA		;set data high
	rjmp rdelay
	

clow:	cbi SENSOR_PORT,PIN_CLk		;set clock low
	rjmp rdelay
	
chigh:	sbi SENSOR_PORT,PIN_CLK		;set clock high

rdelay:	push temp1			;1 msek delay to allow the lines to settle
	push temp2
	ldi temp1,1
	rcall g_wms
	pop temp2
	pop temp1
	ret

;-------PAUSE----------
g_wms:	ldi temp2,250	;wait zl msec  (at 4MHz clock)
g_wmsa:	rcall g_wmsb
	nop
	nop
	nop
	nop
	nop
	dec temp2
	brne g_wmsa
	dec temp1
	brne g_wms
g_wmsb:	ret	



readsh11:
	clr trt
	
	clr rhrt
	cbi SENSOR_PORT,PIN_CLK		;init clock line
	sbi SENSOR_DDR,PIN_CLK
	
	cbi SENSOR_PORT,PIN_DATA		;init data line
	cbi SENSOR_DDR,PIN_DATA
	
	rcall rdelay
	
	
	ldi temp1 ,10		;connection reset sequence
	
re1:	rcall chigh
	rcall clow
	dec temp1
	brne re1


	rcall chigh		;transmission start
	rcall dlow
	rcall clow
	rcall chigh
	rcall dhigh
	rcall clow	

	ldi temp1,0b00000101	;send byte: command for 'check RH'
	rcall sendbyte
;---------	
				;wait for sensor ready 
re3:	ldi temp1,2
	rcall g_wms
	sbis SENSOR_PIN,PIN_DATA
	rjmp re2
	inc rhrt
	brne re3
	
re4:	rjmp reto		;sensor timed out (512 ms)
	
re2:	rcall readbyte		;read one byte

	rcall dlow		;acknowlegde
	rcall chigh
	rcall clow
	rcall dhigh	
	
	rcall readbyte		;readbyte
	
	mov rhl,temp2
	
	rcall chigh		;skip acknowlegde to end communication
	rcall clow
	
	
				;--------read temperature
				

	rcall chigh		;transmission start
	rcall dlow
	rcall clow
	rcall chigh
	rcall dhigh
	rcall clow
		
		
	ldi temp1,0b00000011	;send byte: command for 'check temp'
	rcall sendbyte
	
				;wait for sensor ready 
rew3:	ldi zl,2
	rcall g_wms
	sbis SENSOR_PIN,PIN_DATA
	rjmp rew2
	inc trt
	brne rew3
	
rew4:	rjmp reto		;sensor timed out (512 ms)
	
rew2:	rcall readbyte		;read one byte

	mov th,temp2

	rcall dlow		;acknowlegde
	rcall chigh
	rcall clow
	rcall dhigh	
	
	rcall readbyte		;readbyte
	
	mov tl,temp2
	
	rcall chigh		;skip acknowlegde to end communication
	rcall clow
	
	ret
	
;-----------time out

reto:	movw temp1,rel
	inc temp1		;increase number of timeouts
	movw rel,temp1

	ret
	
	
	
	

;------------	
	
readbyte:
	ldi temp1,8
	ldi	temp2,0
ree1:	clc			;Read bit
	sbic SENSOR_PIN,PIN_DATA		
	sec			
		
	rol temp2
	
	rcall chigh
	rcall clow
	
	dec temp1
	brne ree1

	ret
	
	

;----------


sendbyte:
	ldi temp2,8		;send 8 bits
se3:	lsl temp1
	brcc se1
	rcall dhigh
	rjmp se2
se1:
	rcall dlow
se2:	
	rcall chigh
	rcall clow
	dec temp2
	brne se3

	rcall dhigh		;release dataline

	clc			;check ack
	sbic SENSOR_PORT,PIN_DATA		;c=0: ack recieved
	sec			;c=1: ack not recieved
	
	push temp2
	in temp2,sreg
	
	rcall chigh		;flush ack bit
	rcall clow
	
	out sreg,temp2
	pop temp2
	
	ret

;----------------calculate temperature---------------------
; tl:th		Temperatur
; temp1 = 4010
; d1 = -40.10	*100  = -4010
; d2 =   0.001	*100  = 1
; Calculation: T = d1+d2*SOt --> T = SOt-Temp1

calchumtemp:
	
	ldi temp1, LOW(Zahl)
    ldi temp2, HIGH(Zahl)
	
	sub   tl, temp1
    sbc   th, temp2
	
	ret
;temperature is now in th:tl in Temperature * 100
	
	

;----------------calculate humidity------------------------
; rhl:rhh	Humidity
; 
; c1 = -2.0468*10^3 = -2046
; c2 =  0.0367*10^3 = 36.7 (in C21 und C22)
; c3 = -1.5955*10^-6*10^3 = 15955/1000000
; Calculation: H = c1+c2*SOrh+c3*SOrh*SOrh
; Calculation here: H = u2-u3-u1
; 						u1 = c1
;						u2 = c21 * (rhl:rhh)+ c22 * (rhl:rhh)/10
;						u3 = (rhl:rhh) * (rhl:rhh) / 1000* c3 / 1000
calchumhum:
	//u1 in (u11:u12):
		ldi temp1, LOW(C1)
		
		ldi	temp2, HIGH(C1)
		
		ldi temp3, 0
		ldi temp4, 0

		ldi temp5, LOW(C23)
		ldi	temp6, HIGH(C23)

		rcall div32

		lds temp1, temp8
		lds temp2, temp9

		sts u11, temp1
		sts u12, temp2

	//u2
	
		//c21*(rhl:rhh)
		ldi	temp1, LOW(C21)
		ldi temp2, HIGH(C21)

		mov temp3, rhl
		mov temp4, rhh

		rcall multi16

		lds temp1, temp8
		lds temp2, temp9
		lds temp3, temp10
		lds temp4, temp11

		sts	u21, temp1
		sts u22, temp2
		sts u23, temp3
		sts u24, temp4

		//c22 * (rhl:rhh)/10
		ldi temp1, LOW(C22)
		ldi temp2, HIGH(C22)

		mov temp3, rhl
		mov temp4, rhh

		rcall multi16

		ldi temp5, LOW(C23)
		ldi	temp6, HIGH(C23)

		lds	temp1, temp8
		lds	temp2, temp9
		lds	temp3, temp10
		lds temp4, temp11

		rcall div32

		//c21+c22
		lds temp5, temp8
		lds temp6, temp9
		lds temp7, temp10
		lds temp1, u21
		lds temp2, u22
		lds temp3, u23
		lds temp4, u24


		rcall add16

		lds	temp1, temp8
		lds	temp2, temp9
		lds	temp3, temp10
		lds temp4, temp11

		ldi temp5, LOW(C23)
		ldi	temp6, HIGH(C23)

		rcall div32

	//u2 fertig
	lds	temp1, temp8
	lds	temp2, temp9
	lds	temp3, temp10
	lds temp4, temp11 
	sts u21, temp1
	sts u22, temp2
	sts u23, temp3
	sts u24, temp4

	//u3
		
		//(rhl:rhh) * (rhl:rhh)
		mov temp1, rhl
		mov temp2, rhh
		mov temp3, rhl
		mov temp4, rhh

		rcall multi16

		lds temp1, temp8
		lds temp2, temp9
		lds temp3, temp10
		lds temp4, temp11

		sts	u31, temp1
		sts u32, temp2
		sts u33, temp3
		sts u34, temp4
		//u3 / C32 
		ldi temp5, LOW(C32)
		ldi temp6, HIGH(C32)

		rcall div32

		lds temp1, temp8
		lds temp2, temp9
		lds temp3, temp10
		lds temp4, temp11

		sts	u31, temp1
		sts u32, temp2
		sts u33, temp3
		sts u34, temp4

		//u3 * c31

		ldi temp3, LOW(C31)
		ldi temp4, HIGH(C31)

		rcall multi16

		lds temp1, temp8
		lds temp2, temp9
		lds temp3, temp10
		lds temp4, temp11

		sts	u31, temp1
		sts u32, temp2
		sts u33, temp3
		sts u34, temp4

		//u3 / c33

		ldi temp5, LOW(C32)
		ldi temp6, HIGH(C32)

		rcall div32

		ldi temp5, LOW(C23)
		ldi temp6, HIGH(C23)

		rcall div32

	//u3 fertig
	lds temp1, temp8
	lds temp2, temp9
	lds temp3, temp10
	lds temp4, temp11

	sts	u31, temp1
	sts u32, temp2
	sts u33, temp3
	sts u34, temp4

	//Endrechnung u2-u3-u1
		//u2-u3
		lds temp4, u24
		lds temp3, u23
		lds temp2, u22
		lds temp1, u21

		lds temp7, u33
		lds temp6, u32
		lds temp5, u31

		rcall sub16

		//(u2-u3)-u1

		lds temp1, temp8
		lds temp2, temp9
		lds temp3, temp10
		lds temp4, temp11
		
		ldi temp7, 0
		lds temp6, u12
		lds temp5, u11

		rcall sub16

		lds rhl, temp8
		lds rhh, temp9
	ret


;----------------Division von 32Bit-----------
;Variable temp1:temp4 werden mit temp5:temp6 dividiert
;Endergebins in temp11:temp8
;

div32:
		ldi temp7, 0
		sts temp8, temp5		//b0
		sts temp9, temp6		//b1
		sts temp10, temp7		//b2
		sts temp11, temp7		//b3
		sts temp12, temp1		//a0
		sts temp13, temp2		//a1
		
		sts temp14, temp3		//a2
		sts temp15, temp4		//a3
		sts temp16, temp7		//t0
		sts temp17, temp7		//t1
		sts temp18, temp7		//t2
		sts temp19, temp7		//t3
		sts temp20, temp7 		//t4


urdiv32:
		sts	temp16, temp5			;T = B
		sts	temp17, temp6
		sts	temp18, temp7
		sts	temp19, temp7
		lsr	temp7					;B / 2
		sts	temp19, temp7
		ror	temp7
		sts	temp18, temp7
		ror	temp6
		sts	temp17, temp6
		ror	temp5
		sts	temp16, temp5

		lds temp7, temp12
		add	temp7, temp5				;A = A + B / 2
		sts temp12, temp7

		lds temp7, temp13
		lds temp6, temp17
		adc	temp7, temp6
		sts temp13, temp7
	
		lds temp7, temp14
		lds temp3, temp18
		adc	temp7, temp3
		sts temp14, temp7
	
		lds temp7, temp15
		lds temp5, temp19
		adc	temp7, temp4
		sts temp15, temp7 
 
; a3..0 = a3..0 / b3..0
; b3..0 = remainde
 
;cycle: max 684
udiv32:
		ldi temp7, 0
		sts temp16, temp7
		sts temp17, temp7
		sts temp18, temp7
		sts temp19, temp7

		ldi	temp5, 32

udi1:	
		lds temp1, temp12
		lds temp2, temp13
		lds temp3, temp14
		lds temp4, temp15
		lsl	temp1
		rol	temp2
		rol	temp3
		rol	temp4
		sts	temp12, temp1
		sts temp13, temp2
		sts temp14, temp3
		sts temp15, temp4

		lds temp1, temp16
		lds temp2, temp17
		lds temp3, temp18
		lds temp4, temp19
		rol	temp1
		rol	temp2
		rol	temp3
		rol	temp4
		sts temp16, temp1
		sts temp17, temp2
		sts temp18, temp3
		sts temp19, temp4

		lds temp7, temp8
		cp	temp1, temp7
		lds temp7, temp9
		cpc	temp2, temp7
		lds temp7, temp10
		cpc	temp3, temp7
		lds temp7, temp11
		cpc	temp4, temp7
		brcs	udi2
		lds temp7, temp8
		sub	temp1, temp7
		sts temp8, temp7
		lds temp7, temp9
		sbc	temp2, temp7
		sts temp9, temp7
		lds temp7, temp10
		sbc	temp3, temp7
		sts temp10, temp7
		lds temp7, temp11
		sbc	temp4, temp7
		sts temp11, temp7
		lds temp7, temp12
		inc	temp7
		sts temp12, temp7
		sts	temp16, temp1
		sts temp17, temp2
		sts temp18, temp3
		sts temp19, temp4
udi2:	
	    dec	temp5
		brne udi1_
		lds temp1, temp12
		lds temp2, temp13
		lds temp3, temp14
		lds temp4, temp15
		sts	temp8, temp1
		sts	temp9, temp2
		sts	temp10, temp3
		sts	temp11, temp4

		ret
udi1_:
		rjmp udi1




;----------------Multiplizieren von 16Bit-----------
;Variable temp1:temp2 und temp3:temp4 werden multipliziert
;Endergebins in temp11:temp8
;
multi16:
		clr temp7
		mul temp2,temp4 ; Multipliziere MSBs
		sts temp10,R0 ; Kopie in obere Ergebnisregister
		sts temp11,R1
		
		mul temp1,temp3 ; Multipliziere LSBs
		sts temp8,R0 ; Kopie in untere Ergebnisregister
		sts temp9,R1
		
		mul temp2,temp3 ; Multipliziere 1M mit 2L
		lds temp5, temp9
		add temp5,R0 ; Addiere zum Ergebnis
		sts temp9, temp5

		lds temp5, temp10
		adc temp5, R1
		sts temp10, temp5
		
		lds temp5, temp11
		adc temp5,temp7 ; Addiere Uebertrag
		sts temp11, temp5

		mul temp1,temp4 ; Multipliziere 1L mit 2M
		lds temp5, temp9
		add temp5,R0 ; Addiere zum Ergebnis
		sts temp9, temp5

		lds temp5, temp10
		adc temp5,R1
		sts temp10, temp5

		lds temp5, temp11
		adc temp5,temp7
		sts temp11, temp5

		ret


;----------------Addition von 24Bit-------------------------
;Variable temp1:temp4 und temp5:temp7 werden addiert
;Endergebins in temp9:temp8
;
add16:
		 ;
	    ; < temp8, temp9, temp10, temp11 > = < temp1, temp2, temp3, temp4 > + < temp5, temp6, temp7, 0 >
    	;
 
	    add temp1, temp5
    	adc temp2, temp6
	    adc temp3, temp7
		ldi temp7, 0
    	adc temp4, temp7
		sts	temp8, temp1
		sts temp9, temp2
		sts temp10, temp3
		sts temp11, temp4

		ret
;----------------Subtraktion von 16Bit----------------------
;Variable temp1:temp2 und temp3:temp4 werden subtrahiert
;Endergebins in temp9:temp8
;

sub16:
		;
 	   	; < temp8, temp9, temp10, temp11 > = < temp1, temp2, temp3, temp4 > - < temp5, temp6, temp7, 0 >
   	 	;
 
    	sub temp1, temp5
   	 	sbc temp2, temp6
   		sbc temp3, temp7
		ldi temp7, 0
   		sbc temp4, temp7
		sts	temp8, temp1
		sts temp9, temp2
		sts temp10, temp3
		sts temp11, temp4

		ret

