;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                 Uhr-Routinen                ;;
;;                 ============                ;;
;;              (c)Philip Ockert               ;;
;;                  				           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "m8def.inc"
.def temp1 = r16
.def temp2 = r17
.def temp3 = r18
.def temp4 = r24
.def temp5 = r27
.def temp6 = r25
.def temp7 = r26

//Flaggenregister (Sleepmode,Flage fr Interrupt,Pumpe)
;0 = Uhrzeit Interrupt
;1 = Temperatur gendert
;2 = Feuchtigkeit gendert
;3 = Taste
;5 = Pumpe
;6 = Sleepmode
.def flag  = r19

//Uhrzeit speicher
.def SubCount = r20
.def Sekunden = r21
.def Minuten  = r22
.def Stunden  = r23

//Pumpe
.def pumpendauer = r6

//Feuchtigkeit
.def feuchtigkeitsgrenze = r7

//Zeitschaltuhr
.def Anfangszeit_Stunde		= r28
.def Anfangszeit_Minute		= r29
.def Enduhrzeit_Stunde		= r30
.def Enduhrzeit_Minute		= r31

//Feuchtigkeitscheck Zeitangabe
.equ Feucht_check = 10

//Buttons
.equ Button = PORTB
.equ ButtonPIN = PINB
.equ ButtonDDR = DDRB
.equ auf 	= 3
.equ ab  	= 0
.equ links	= 2
.equ rechts	= 1

//Relais
.equ Relais_Port_Lampen = PORTB
.equ Relais_DDR_Lampen = DDRB
.equ Relais_Port_Pumpe = PORTC
.equ Relais_DDR_Pumpe = DDRC
.equ UV_Lampe = PB4
.equ Waerme_Lampe = PB5
.equ Pumpe = PC0

//LCD-Display
.equ Zeile1 = 0x80
.equ Zeile2 = 0xC0
.equ Zeile3 = 0x94
.equ Zeile4 = 0xD4

.org 0x0000
           rjmp    main             ; Reset Handler
.org OC1Aaddr
           rjmp    get8key   ; Timer Compare Handler
.org OVF1addr						
      	   in     save_sreg, SREG

//Include Dateien
.include "entprell.asm"
.include "Uhr_timer.asm"
.include "lcd-routines.asm"
.include "SHT71-Routinen.asm"
main:	
		
		ldi     temp1, HIGH(RAMEND)
        out     SPH, temp1
        ldi     temp1, LOW(RAMEND)  ; Stackpointer initialisieren
        out     SPL, temp1
		rcall	register_init
		rcall	entprell_init
		rcall	uhr_init
		sbi		Relais_DDR_Lampen, UV_Lampe		;Relais_init
		cbi		Relais_Port_Lampen, UV_Lampe
		sbi		Relais_DDR_Lampen, Waerme_Lampe
		cbi		Relais_Port_Lampen, Waerme_Lampe
		sbi		Relais_DDR_Pumpe, Pumpe
		cbi		Relais_Port_Pumpe, Pumpe

loop:
        SBRC    flag,0
		rcall   lcd_main_anzeigen				               ; Flag im Interrupt gesetzt?Ja? Uhrzeit erneuern
		rcall	Menue_einstellungen
		rcall	Beleuchtung_kontrolle						   ;End- oder Startzeit erreicht?
		rcall	Pumpe_manuel
		rcall	check_Sensor
        rjmp 	loop


//Button entprellen und Gedrckt kontrolle
Menue_einstellungen:
		cbr		flag, 8
		SBRS	key_press,rechts
		ret
		clr    key_press
		SBRC	flag, 5			//Pumpe ist nicht an?
		ret
		rcall	Menue_Uhrzeit	//Uhrzeiteinstellen
		rcall	lcd_clear
		SBRC	flag, 3
		ret
		rcall	Menue_Zeitschaltuhr
		rcall	lcd_clear
		SBRC	flag, 3
		ret
		rcall	Menue_Pumpendauer
		rcall	lcd_clear
		SBRC	flag, 3
		ret
		rcall	Menue_Feuchtigkeit
		rcall	lcd_clear
		ret

Menue_Uhrzeit:
		rcall	lcd_clear
		rcall	lcd_home

		ldi		temp1, 'U'		//Uhrzeit Einstellung
		rcall	lcd_data
		ldi		temp1, 'h'
		rcall	lcd_data
		ldi		temp1, 'r'
		rcall	lcd_data
		ldi		temp1, 'z'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'i'
		rcall	lcd_data
		ldi		temp1, 't'
		rcall	lcd_data
		ldi		temp1, ' '
		rcall	lcd_data
		ldi		temp1, 'E'
		rcall	lcd_data
		ldi		temp1, 'i'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, 's'
		rcall	lcd_data
		ldi		temp1, 't'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'l'
		rcall	lcd_data
		ldi		temp1, 'l'
		rcall	lcd_data
		ldi		temp1, 'u'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, 'g'
		rcall	lcd_data

		rcall	Stunden_aendern
		rcall	Minuten_aendern
		SBRC	key_press,links
		sbr		flag, 8
		clr		key_press
		ret

Menue_Zeitschaltuhr:
		rcall	lcd_clear
		rcall	lcd_home

		ldi		temp1, 'Z'		//Uhrzeit Einstellung
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'i'
		rcall	lcd_data
		ldi		temp1, 't'
		rcall	lcd_data
		ldi		temp1, 's'
		rcall	lcd_data
		ldi		temp1, 'c'
		rcall	lcd_data
		ldi		temp1, 'h'
		rcall	lcd_data
		ldi		temp1, 'a'
		rcall	lcd_data
		ldi		temp1, 'l'
		rcall	lcd_data
		ldi		temp1, 't'
		rcall	lcd_data
		ldi		temp1, 'u'
		rcall	lcd_data
		ldi		temp1, 'h'
		rcall	lcd_data
		ldi		temp1, 'r'
		rcall	lcd_data
		ldi		temp1, ':'
		rcall	lcd_data
		ldi		temp1, ' '

		ldi		temp1,Zeile2
		rcall	lcd_command
		ldi		temp1, 'A'		
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, 'f'
		rcall	lcd_data
		ldi		temp1, 'a'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, 'g'
		rcall	lcd_data
		ldi		temp1, ':'
		rcall	lcd_data
		ldi		temp1, ' '

		rcall	Anfangszeit_Stunden_einstellen
		rcall	Anfangszeit_Minuten_einstellen
		
		ldi		temp1,Zeile2
		rcall	lcd_command
		ldi		temp1, 'E'		
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, 'd'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, ':'
		rcall	lcd_data
		ldi		temp1, ' '
		rcall   lcd_data
		ldi		temp1, ' '
		rcall   lcd_data

		rcall	Enduhrzeit_Stunden_einstellen
		rcall	Enduhrzeit_Minuten_einstellen
		SBRC	key_press,links
		sbr		flag, 8
		clr		key_press
		ret

Menue_Pumpendauer:
		rcall	lcd_clear
		rcall	lcd_home

		ldi		temp1, 'P'		//Uhrzeit Einstellung
		rcall	lcd_data
		ldi		temp1, 'u'
		rcall	lcd_data
		ldi		temp1, 'm'
		rcall	lcd_data
		ldi		temp1, 'p'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, 'd'
		rcall	lcd_data
		ldi		temp1, 'a'
		rcall	lcd_data
		ldi		temp1, 'u'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'r'
		rcall	lcd_data
		ldi		temp1, ':'
		rcall	lcd_data
		ldi		temp1, ' '

		rcall	Pumpendauer_einstellen
		SBRC	key_press,links
		sbr		flag, 8
		clr		key_press
		ret

Menue_Feuchtigkeit:
		rcall	lcd_clear
		rcall	lcd_home

		ldi		temp1, 'F'		//Uhrzeit Einstellung
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'u'
		rcall	lcd_data
		ldi		temp1, 'c'
		rcall	lcd_data
		ldi		temp1, 'h'
		rcall	lcd_data
		ldi		temp1, 't'
		rcall	lcd_data
		ldi		temp1, 'i'
		rcall	lcd_data
		ldi		temp1, 'g'
		rcall	lcd_data
		ldi		temp1, 'k'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'i'
		rcall	lcd_data
		ldi		temp1, 't'
		rcall	lcd_data
		ldi		temp1, 's'
		rcall	lcd_data
		ldi		temp1, 'g'
		rcall	lcd_data
		ldi		temp1, 'r'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, 'z'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, ':'

		rcall	Feuchtigkeitsgrenze_einstellen
		clr		key_press
		ret

Stunden_aendern:
		ldi	  	temp1,Zeile2			;Cursor in Zeile 2
		rcall  	lcd_command
		ldi		temp1, 'S'
		rcall	lcd_data
		ldi		temp1, 't'
		rcall	lcd_data
		ldi		temp1, 'u'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, 'd'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, ':'
		rcall	lcd_data
		ldi		temp1, ' '
		rcall	lcd_data

		mov		temp1, Stunden
		rcall	lcd_number
		ldi		temp1, 0xC9
		rcall	lcd_command

Uhrzeit_Stunden:
		SBRC	key_press,auf
		rcall	Uhrzeit_Stunden_auf
		SBRC	key_press,ab
		rcall	Uhrzeit_Stunden_ab
		SBRS	key_press,rechts
	    rjmp	Uhrzeit_Stunden
		clr     key_press
		ret

Uhrzeit_Stunden_auf:
		clr     key_press
		inc		Stunden
		ldi		temp1, 24
		cp		temp1, Stunden
		BREQ	Stunden_loeschen
		ldi		temp1, 0xC9
		rcall	lcd_command
		mov		temp1, Stunden
		rcall	lcd_number
		ret

Uhrzeit_Stunden_ab:
		clr     key_press
		ldi		temp1, 0
		cp		temp1, Stunden
		BREQ	Stunden_24
		dec		Stunden
		ldi		temp1, 0xC9
		rcall	lcd_command
		mov		temp1, Stunden
		rcall	lcd_number
		ret

Stunden_loeschen:
		clr		Stunden
		ret

Stunden_24:
		ldi		Stunden, 24
		ret

Minuten_aendern:
		ldi	  	temp1,Zeile2			;Cursor in Zeile 2
		rcall  	lcd_command
		ldi		temp1, 'M'
		rcall	lcd_data
		ldi		temp1, 'i'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, 'u'
		rcall	lcd_data
		ldi		temp1, 't'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, ':'
		rcall	lcd_data
		ldi		temp1, ' '
		rcall	lcd_data

		ldi		temp1, 0xC9
		rcall	lcd_command
		mov		temp1, Minuten
		rcall	lcd_number

Uhrzeit_Minuten:
		SBRC	key_press,auf
		rcall	Uhrzeit_Minuten_auf
		SBRC	key_press,ab
		rcall	Uhrzeit_Minuten_ab
		SBRC	key_press,links
		ret
		SBRS	key_press,rechts
	    rjmp	Uhrzeit_Minuten
		ret

Uhrzeit_Minuten_auf:
		clr     key_press
		inc		Minuten
		ldi		temp1, 61
		cp		temp1, Minuten
		BREQ	Minuten_loeschen
		ldi		temp1, 0xC9
		rcall	lcd_command
		mov		temp1, Minuten
		rcall	lcd_number
		ret
Uhrzeit_Minuten_ab:
		clr     key_press
		ldi		temp1, 0
		cp		temp1, Minuten
		BREQ	Minuten_60
		dec		Minuten
		ldi		temp1, 0xC9
		rcall	lcd_command
		mov		temp1, Minuten
		rcall	lcd_number
		ret
Minuten_loeschen:
		clr		Minuten
		ret
Minuten_60:
		ldi		Minuten, 60
		ret

Anfangszeit_Stunden_einstellen:
		ldi	  	temp1,Zeile3			;Cursor in Zeile 2
		rcall  	lcd_command
		ldi		temp1, 'S'
		rcall	lcd_data
		ldi		temp1, 't'
		rcall	lcd_data
		ldi		temp1, 'u'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, 'd'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, ':'
		rcall	lcd_data
		ldi		temp1, ' '
		rcall	lcd_data

		mov		temp1, Anfangszeit_Stunde
		rcall	lcd_number
		ldi		temp1, 0xC9
		rcall	lcd_command

Anfangszeit_Stunden:
		SBRC	key_press,auf
		rcall	Anfangszeit_Stunden_auf
		SBRC	key_press,ab
		rcall	Anfangszeit_Stunden_ab
		SBRS	key_press,rechts
	    rjmp	Anfangszeit_Stunden
		clr     key_press
		ret
Anfangszeit_Stunden_auf:
		clr     key_press
		inc		Anfangszeit_Stunde
		ldi		temp1, 24
		cp		temp1, Anfangszeit_Stunde
		BREQ	Anfangszeit_Stunden_loeschen
		ldi		temp1, 0x9D
		rcall	lcd_command
		mov		temp1, Anfangszeit_Stunde
		rcall	lcd_number
		ret

Anfangszeit_Stunden_ab:
		clr     key_press
		ldi		temp1, 0
		cp		temp1, Anfangszeit_Stunde	
		BREQ	Anfangszeit_Stunden_24
		dec		Anfangszeit_Stunde	
		ldi		temp1, 0x9D
		rcall	lcd_command
		mov		temp1, Anfangszeit_Stunde
		rcall	lcd_number
		ret

Anfangszeit_Stunden_loeschen:
		clr		Anfangszeit_Stunde
		ret

Anfangszeit_Stunden_24:
		ldi		Anfangszeit_Stunde, 24
		ret
Anfangszeit_Minuten_einstellen:
		ldi	  	temp1,Zeile3			;Cursor in Zeile 2
		rcall  	lcd_command
		ldi		temp1, 'M'
		rcall	lcd_data
		ldi		temp1, 'i'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, 'u'
		rcall	lcd_data
		ldi		temp1, 't'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, ':'
		rcall	lcd_data
		ldi		temp1, ' '
		rcall	lcd_data

		ldi		temp1, 0x9D
		rcall	lcd_command
		mov		temp1, Anfangszeit_Minute
		rcall	lcd_number

Anfangszeit_Minuten:
		SBRC	key_press,auf
		rcall	Anfangszeit_Minuten_auf
		SBRC	key_press,ab
		rcall	Anfangszeit_Minuten_ab
		SBRS	key_press,rechts
	    rjmp	Anfangszeit_Minuten
		ret

Anfangszeit_Minuten_auf:
		clr     key_press
		inc		Anfangszeit_Minute
		ldi		temp1, 61
		cp		temp1, Anfangszeit_Minute
		BREQ	Anfangszeit_Minuten_loeschen
		ldi		temp1, 0x9D
		rcall	lcd_command
		mov		temp1, Anfangszeit_Minute
		rcall	lcd_number
		ret
Anfangszeit_Minuten_ab:
		clr     key_press
		ldi		temp1, 0
		cp		temp1, Anfangszeit_Minute
		BREQ	Anfangszeit_Minuten_60
		dec		Anfangszeit_Minute
		ldi		temp1, 0x9D
		rcall	lcd_command
		mov		temp1, Anfangszeit_Minute
		rcall	lcd_number
		ret
Anfangszeit_Minuten_loeschen:
		clr		Anfangszeit_Minute
		ret
Anfangszeit_Minuten_60:
		ldi		Anfangszeit_Minute, 60
		ret

Enduhrzeit_Stunden_einstellen:
		ldi	  	temp1,Zeile3			;Cursor in Zeile 2
		rcall  	lcd_command
		ldi		temp1, 'S'
		rcall	lcd_data
		ldi		temp1, 't'
		rcall	lcd_data
		ldi		temp1, 'u'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, 'd'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, ':'
		rcall	lcd_data
		ldi		temp1, ' '
		rcall	lcd_data

		mov		temp1, Enduhrzeit_Stunde
		rcall	lcd_number
		ldi		temp1, 0xC9
		rcall	lcd_command
		clr		key_press

Enduhrzeit_Stunden:
		SBRC	key_press,auf
		rcall	Enduhrzeit_Stunden_auf
		SBRC	key_press,ab
		rcall	Enduhrzeit_Stunden_ab
		SBRS	key_press,rechts
	    rjmp	Enduhrzeit_Stunden
		clr     key_press
		ret
Enduhrzeit_Stunden_auf:
		clr     key_press
		inc		Enduhrzeit_Stunde
		ldi		temp1, 24
		cp		temp1, Enduhrzeit_Stunde
		BREQ	Enduhrzeit_Stunden_loeschen
		ldi		temp1, 0x9D
		rcall	lcd_command
		mov		temp1, Enduhrzeit_Stunde
		rcall	lcd_number
		ret

Enduhrzeit_Stunden_ab:
		clr     key_press
		ldi		temp1, 0
		cp		temp1, Enduhrzeit_Stunde	
		BREQ	Enduhrzeit_Stunden_24
		dec		Enduhrzeit_Stunde	
		ldi		temp1, 0x9D
		rcall	lcd_command
		mov		temp1, Enduhrzeit_Stunde
		rcall	lcd_number
		ret

Enduhrzeit_Stunden_loeschen:
		clr		Enduhrzeit_Stunde
		ret

Enduhrzeit_Stunden_24:
		ldi		Enduhrzeit_Stunde, 24
		ret
Enduhrzeit_Minuten_einstellen:
		ldi	  	temp1,Zeile3			;Cursor in Zeile 2
		rcall  	lcd_command
		ldi		temp1, 'M'
		rcall	lcd_data
		ldi		temp1, 'i'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, 'u'
		rcall	lcd_data
		ldi		temp1, 't'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'n'
		rcall	lcd_data
		ldi		temp1, ':'
		rcall	lcd_data
		ldi		temp1, ' '
		rcall	lcd_data

		ldi		temp1, 0x9D
		rcall	lcd_command
		mov		temp1, Enduhrzeit_Minute
		rcall	lcd_number

Enduhrzeit_Minuten:
		SBRC	key_press,auf
		rcall	Enduhrzeit_Minuten_auf
		SBRC	key_press,ab
		rcall	Enduhrzeit_Minuten_ab
		SBRC	key_press,links
		ret
		SBRS	key_press,rechts
	    rjmp	Enduhrzeit_Minuten
		ret

Enduhrzeit_Minuten_auf:
		clr     key_press
		inc		Enduhrzeit_Minute
		ldi		temp1, 61
		cp		temp1, Enduhrzeit_Minute
		BREQ	Enduhrzeit_Minuten_loeschen
		ldi		temp1, 0x9D
		rcall	lcd_command
		mov		temp1, Enduhrzeit_Minute
		rcall	lcd_number
		ret
Enduhrzeit_Minuten_ab:
		clr     key_press
		ldi		temp1, 0
		cp		temp1, Enduhrzeit_Minute
		BREQ	Enduhrzeit_Minuten_60
		dec		Enduhrzeit_Minute
		ldi		temp1, 0x9D
		rcall	lcd_command
		mov		temp1, Enduhrzeit_Minute
		rcall	lcd_number
		ret
Enduhrzeit_Minuten_loeschen:
		clr		Enduhrzeit_Minute
		ret
Enduhrzeit_Minuten_60:
		ldi		Enduhrzeit_Minute, 60
		ret

Pumpendauer_einstellen:
		ldi	  	temp1,Zeile2			;Cursor in Zeile 2
		rcall  	lcd_command
		ldi		temp1, 'D'
		rcall	lcd_data
		ldi		temp1, 'a'
		rcall	lcd_data
		ldi		temp1, 'u'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'r'
		rcall	lcd_data
		ldi		temp1, ':'
		rcall	lcd_data
		ldi		temp1, ' '
		rcall	lcd_data

		mov		temp1, Pumpendauer
		rcall	lcd_number
		ldi		temp1, 0xC7
		rcall	lcd_command
		clr		key_press

Pumpendauer_loop:
		SBRC	key_press,auf
		rcall	Pumpendauer_auf
		SBRC	key_press,ab
		rcall	Pumpendauer_ab
		SBRC	key_press,links
		ret
		SBRS	key_press,rechts
	    rjmp	Pumpendauer_loop
		clr     key_press
		ret
Pumpendauer_auf:
		clr     key_press
		inc		Pumpendauer
		ldi		temp1, 61
		cp		temp1, Pumpendauer
		BREQ	Pumpendauer_loeschen
		ldi		temp1, 0xC7
		rcall	lcd_command
		mov		temp1, Pumpendauer
		rcall	lcd_number
		ret

Pumpendauer_ab:
		clr     key_press
		ldi		temp1, 0
		cp		temp1, Pumpendauer	
		BREQ	Pumpendauer_60
		dec		Pumpendauer	
		ldi		temp1, 0xC7
		rcall	lcd_command
		mov		temp1, Pumpendauer
		rcall	lcd_number
		ret

Pumpendauer_loeschen:
		clr		Pumpendauer
		ret

Pumpendauer_60:
		ldi		temp1, 60
		mov		Pumpendauer, temp1
		ret

Feuchtigkeitsgrenze_einstellen:
		ldi	  	temp1,Zeile2			;Cursor in Zeile 2
		rcall  	lcd_command
		ldi		temp1, 'F'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'u'
		rcall	lcd_data
		ldi		temp1, 'c'
		rcall	lcd_data
		ldi		temp1, 'h'
		rcall	lcd_data
		ldi		temp1, 't'
		rcall	lcd_data
		ldi		temp1, 'i'
		rcall	lcd_data
		ldi		temp1, 'g'
		rcall	lcd_data
		ldi		temp1, 'k'
		rcall	lcd_data
		ldi		temp1, 'e'
		rcall	lcd_data
		ldi		temp1, 'i'
		rcall	lcd_data
		ldi		temp1, 't'
		rcall	lcd_data
		ldi		temp1, ':'
		rcall	lcd_data
		ldi		temp1, ' '
		rcall	lcd_data

		mov		temp1, feuchtigkeitsgrenze
		rcall	lcd_number
		ldi		temp1, '%'
		rcall	lcd_data
		ldi		temp1, 0xCE
		rcall	lcd_command
		clr		key_press

feuchtigkeitsgrenze_loop:
		SBRC	key_press,auf
		rcall	feuchtigkeitsgrenze_auf
		SBRC	key_press,ab
		rcall	feuchtigkeitsgrenze_ab
		SBRS	key_press,rechts
	    rjmp	feuchtigkeitsgrenze_loop
		clr     key_press
		ret
feuchtigkeitsgrenze_auf:
		clr     key_press
		inc		feuchtigkeitsgrenze
		ldi		temp1, 100
		cp		temp1, feuchtigkeitsgrenze
		BREQ	feuchtigkeitsgrenze_loeschen
		ldi		temp1, 0xCE
		rcall	lcd_command
		mov		temp1, feuchtigkeitsgrenze
		rcall	lcd_number
		ret

feuchtigkeitsgrenze_ab:
		clr     key_press
		ldi		temp1, 0
		cp		temp1, feuchtigkeitsgrenze	
		BREQ	feuchtigkeitsgrenze_100
		dec		feuchtigkeitsgrenze
		ldi		temp1, 0xCE
		rcall	lcd_command
		mov		temp1, feuchtigkeitsgrenze
		rcall	lcd_number
		ret

feuchtigkeitsgrenze_loeschen:
		clr		feuchtigkeitsgrenze
		ret

feuchtigkeitsgrenze_100:
		ldi		temp1, 100
		mov		feuchtigkeitsgrenze, temp1
		ret
		
register_init:
		ldi		temp1, 0
		sts		Feuchtig_flag, temp1
		sts		Sekundencheck, temp1
		mov		flag, temp1
		mov		SubCount, temp1
		mov		Sekunden, temp1
		mov		Minuten, temp1
		mov		Stunden, temp1
		mov		Anfangszeit_Stunde, temp1
		mov		Anfangszeit_Minute, temp1
		ldi		Enduhrzeit_Stunde, 23
		ldi	 	Enduhrzeit_Minute, 00
		ldi		temp1, 0
		mov		feuchtigkeitsgrenze, temp1
		mov		rhl, temp1
		mov		rhh, temp1
		mov		tl, temp1
		mov		th, temp1
		ldi		temp1, 10
		mov		pumpendauer, temp1
		sbr		Enduhrzeit_Stunde,23
		ret
Beleuchtung_kontrolle:
		clr		key_press
		SBRS    flag,1
		rcall	anfangszeit_kontrolle
		SBRC    flag,1
		rcall	endzeit_kontrolle
		ret

anfangszeit_kontrolle:
		cpc		Anfangszeit_Stunde, Stunden
		BREQ	Minutenkontrolle
		ret
Minutenkontrolle:
		cpc		Anfangszeit_Minute, Minuten
		BREQ	Einschalten
		ret
Einschalten:
		sbis 	Relais_Port_Lampen, UV_Lampe
		sbi		Relais_Port_Lampen, UV_Lampe
		sbis	Relais_Port_Lampen, Waerme_Lampe
		sbi		Relais_Port_Lampen, Waerme_Lampe
		sbr		flag,2
		ret


endzeit_kontrolle: 
		cpc		Enduhrzeit_Stunde, Stunden
		BREQ	Minutenkontrolle2
		ret
Minutenkontrolle2: 
		cpc		Enduhrzeit_Minute, Minuten
		BREQ	Ausschalten
		ret
Ausschalten:
		sbic 	Relais_Port_Lampen, UV_Lampe
		cbi		Relais_Port_Lampen, UV_Lampe
		sbic	Relais_Port_Lampen, Waerme_Lampe
		cbi		Relais_Port_Lampen, Waerme_Lampe
		cbr		flag,2
		rcall	Pumpe_an
		ret
Pumpe_manuel:
		SBRC	key_press,links
		rcall	Pumpe_an
		cbi		Relais_Port_Pumpe, Pumpe
		ret
Pumpe_an:
		sbi		Relais_Port_Pumpe, Pumpe
		mov		temp1, Sekunden
		mov		temp2, pumpendauer
		add		temp1,temp2
		ldi		temp2, 60
		cp		temp1, temp2
		BRSH	Pumpe_setzen
Pumpe_loop:
		cp		temp1,Sekunden
		brne	Pumpe_loop
		ret
Pumpe_setzen:
		ldi		temp2, 60
		sub		temp1, temp2
		ret
check_Sensor:
		lds		temp1, Feuchtig_flag
		SBRC	temp1,0
		rcall	Abruf_Sensor

		ret
Abruf_Sensor:
		ldi		temp1, 0
		sts		Feuchtig_flag, temp1
		rcall	readsh11
		rcall	calchumtemp
		rcall	calchumhum
		ret 

		.DSEG		
u11:   	.BYTE  1
u12:	.BYTE  2    
u21:    .BYTE  3
u22:    .BYTE  4
u23:    .BYTE  5
u24:    .BYTE  6        
u31:	.BYTE  7
u32:	.BYTE  8
u33:	.BYTE  9
u34:	.BYTE  10

temp8:	.BYTE  11
temp9:  .BYTE  12
temp10: .BYTE  13
temp11: .BYTE  14
temp12: .BYTE  15
temp13: .BYTE  16
temp14: .BYTE  17
temp15: .BYTE  18
temp16: .BYTE  19
temp17: .BYTE  20
temp18: .BYTE  21
temp19: .BYTE  22
temp20: .BYTE  23

Sekundencheck: .BYTE  24
Feuchtig_flag: .BYTE  25
